/*==============================================================================


       main.cpp - addon file to main


==============================================================================*/
#include "sys.h"         // eBookMan include files
#include "gui.h"

extern "C"
{
#include "ereader_hostio.h"
#include "evnt_fun.h"
#include "lcd.h"
#include "timer.h"
#include "ereader_hostio.h"
#include "piezo.h"
#include "ebm_object.h"
#include "stdlib.h"
#include "string.h"
}

#include "rs232.h"
#define STX 2
#define ETX 3

#include "main_fff.h"
#include "enter_name_fff.h"

extern const IMAGE b_space_bmp;
extern const IMAGE w_space_bmp;

extern const IMAGE b_man_bmp;
extern const IMAGE w_man_bmp;
extern const IMAGE b_select_bmp;
extern const IMAGE w_select_bmp;

extern const IMAGE b_king_bmp;
extern const IMAGE w_king_bmp;
extern const IMAGE b_king_select_bmp;
extern const IMAGE w_king_select_bmp;

enum
{
  MainWindow_Board=110,
  MainWindow_Black_label=150,
  MainWindow_White_label,
};

char *Help_text = "Classic game of checkers.  You play white.
Jog wheel will toggle sound on and off.
EBM plays black or you may use the menu item to play against a friend.
Connect 2 EBMs and play head-to-head by selecting the 'link' option in the menu.

'crossover' USB-Cable:
Left Plug    Right Plug
1 --(External +5V) -- 1
2 --(pull-up* +5V) -- 3
3 --(pull-up* +5V) -- 2
4 --(External GND) -- 4

+5V also used to power the units.
*pull-up should be done with a 10k resistor and diode.

Have fun.
********************
WhitWareInc Software [whitwareinc@yahoo.com]
Version 2.02
"__DATE__"-"__TIME__"
";

#define TIMER_SPEED	500

#define MAX_PLAYERS	2
#define MAX_MATRIX	12
#define MAX_COLUMNS	10
#define MAX_ROWS	10

enum {
	MSG_GAME_LOOP=MSG_USER+1,
	MSG_LINK_PEN_DOWN,
};

enum {
	GS_SELECT = 0,
	GS_MOVE,
};

MainWindow *f_MainWindow;
EnterNameWindow *f_EnterNameWindow;

CTable *Board;

char Title_bar[50]="";
char Link_str[50]="";
char Black_label_str[50]="";
char White_label_str[50]="";
char Color[2][6] = {
	"White",
	"Black",
};
enum {
	HUMAN = 0,
	EBM,
};

U16 Game_score;
U16 From_c, From_r;
U16 Do_move;
U16 Move_c[4], Move_r[4], Num_moves, Was_jump[4], Jump_c[4], Jump_r[4];
U16 Num_whites, Num_blacks, Black_player;
U16 First_c, First_r;

U16 Running, Current_player, Num_players, Num_rows, Num_columns, Game_state;
U16 Player;
U16 Mult_jump;

CIcon *Table_obj[MAX_MATRIX][MAX_MATRIX];
CLabel *Black_label, *White_label;

U8 Sound;

char You_won_str[50];

typedef struct score_struct {U16 score; char name[50];};

void MainWindow::HighScore (U16 score, char *name_p)
{
	struct score_struct *high_scores, tmp_score;
	size_t max_size=ebo_roundup(sizeof(score_struct)*10);

	static const ebo_name_t file_name = { "","WhitWare","Checkers","ini" };
	char *file_ptr=(char *)ebo_roundup(OS_availaddr);
	OS_availaddr=(U32)file_ptr + max_size;

	U16 num_scores, i, j;
	char high_scores_str[500];
	
	if (ebo_mapin(&file_name,0,file_ptr,&max_size,1) < 0)
	{
		if ( ebo_new(&file_name,max_size,1) < 0)
      			return;
		else if (ebo_mapin(&file_name,0,file_ptr,&max_size,1) < 0)
			return;
	}
	high_scores = (score_struct *)file_ptr;
	num_scores = 0;
	high_scores_str[0] = 0;
	while ( high_scores[num_scores].score )
	{
		num_scores++;
	}

	if(score)
	{
		if(num_scores > 9)
			num_scores = 9;
		high_scores[num_scores].score = score;
		strcpy(high_scores[num_scores].name, name_p);
		SetFocus(f_MainWindow);
		num_scores++;
	}
	// qsort(high_scores, (size_t)num_scores, (size_t)sizeof(score_struct), (int (*)(...))ScoreCmp);
	for (i = 0; i < num_scores; i++)
	{
		for (j = i+1; j < num_scores; j++)
			if(high_scores[j].score < high_scores[i].score)
			{
				tmp_score = high_scores[i];
				high_scores[i] = high_scores[j];
				high_scores[j] = tmp_score;
			}
	}		
	sprintf(&high_scores_str[strlen(high_scores_str)],"*************\n  High Scores\n*************\n");
	for (i = 0; i < num_scores; i++)
	{
		sprintf(&high_scores_str[strlen(high_scores_str)],"%6d | %s\n",
			high_scores[i].score, high_scores[i].name);
	}

	GUI_Alert(ALERT_INFO, high_scores_str);

	ebo_unmap(file_ptr, max_size);
}

/***************************************************************************/
/***************************************************************************/
/* Comm IO sample app 	                                                   */
/***************************************************************************/
/***************************************************************************/
void do_open()
{
	if (RS232_Open(RS232_PORTID_0,RS232_BAUD_57600,4096) 
						== RS232_NO_ERROR) {
		strcpy(Link_str, " [Linked]");
	} else {
		strcpy(Link_str, "");
	}
}

void do_close()
{
	RS232_Close(RS232_PORTID_0);
	strcpy(Link_str, "");
}

char Rx_buf[4096]="";

void do_read()
{
	U16 bytes_read,i, start;
	char buf[1024];
	S32 data;

	/* I'm ignoring errors here for simplicity */
	RS232_GetBytes(RS232_PORTID_0, (U8 *)&buf, 1024, &bytes_read, 100 /* 500 */);
	buf[bytes_read]= 0;	//Zero-terminate the string
	strcat(Rx_buf, buf);
	i = start = 0;
	do
	{
		if(Rx_buf[i] == STX)
			start = i+1;
		if((start) && (Rx_buf[i] == ETX))
		{
			Rx_buf[i] = 0;
			data = atol((const char *)&Rx_buf[start]);
            		GUI_EventMessage( (MSG_TYPE)(MSG_LINK_PEN_DOWN), f_MainWindow, data );
			start = 0;
		}
		i++;
	} while (Rx_buf[i]);
	if(start)
	{
		start--;
		i = 0;
		while (Rx_buf[start])
		{
			Rx_buf[i] = Rx_buf[start];
			i++;
			start++;
		}
		Rx_buf[i] = 0;
	}
	else
		Rx_buf[0] = 0;
}

void do_write(char *msg)
{
U16 i, len;

	len = strlen((const char *)msg);
	for( i = len; i >=1; i--)
		msg[i] = msg[i-1];
	msg[0] = STX;
	msg[len+1] = ETX;
	msg[len+2] = 0; 
	RS232_SendBytes(RS232_PORTID_0, (U8 *)msg, len+2);
}


//--------------------------------------------------------------------------------

void BoardSetup( void )
{
	U16 c, r;

	for (r = 0; r < Num_rows; r++)
	{
		for (c = 0; c < Num_columns; c++)
		{
			if ((!(r%2) && !(c%2)) || ((r%2) && (c%2)))
				if (r < 3)
					Table_obj[r][c]->SetImage(&b_man_bmp);
				else if (r > 6)
					Table_obj[r][c]->SetImage(&w_man_bmp);
				else
					Table_obj[r][c]->SetImage(&b_space_bmp);
			else
				Table_obj[r][c]->SetImage(&w_space_bmp);
		}
	}
}

void NewGame ( void )
{
	Current_player = 0;
	Game_score = 0;
	Game_state = GS_SELECT;
	Player = 0;
	Num_whites = 15;
	Num_blacks = 15;
	Mult_jump = 0;

	BoardSetup();
	Board->Draw();

	sprintf(Black_label_str,"Black pieces - %d", Num_blacks);
	Black_label->SetLabel(Black_label_str);
	sprintf(White_label_str,"White pieces - %d", Num_whites);
	White_label->SetLabel(White_label_str);
	
	sprintf(Title_bar,"%s - Select%s", Color[Player], Link_str);
	f_MainWindow->SetTitle(Title_bar);
	Running = 1;
}


//------------------------------------------------------------------------------
void EnterNameWindow::Close( void )
{
	char *name_p;

	name_p = f_EnterNameWindow->EnterName->GetText();
	f_MainWindow->HighScore(Game_score, name_p);
	free(name_p);

	return CWindow::Close();
}

U16 WhitePiece (const IMAGE *tmp_image)
{
	if ((tmp_image == &w_man_bmp) || (tmp_image == &w_king_bmp))
		return 1;
	return 0;
}

U16 BlackPiece (const IMAGE *tmp_image)
{
	if ((tmp_image == &b_man_bmp) || (tmp_image == &b_king_bmp))
		return 1;
	return 0;
}

U16 CanJump ( U16 Player, U16 c, U16 r)
{
const IMAGE *tmp_image;
U16 ret_val = 0;
						
	Num_moves = 0;
	tmp_image = Table_obj[r][c]->GetImage();
	if((!Player) || (tmp_image == &b_king_bmp)) 
	{
		if((c >= 2) && (r >= 2))
		{
			tmp_image = Table_obj[r-1][c-1]->GetImage();
			if(((!Player) && (BlackPiece(tmp_image))) || ((Player) && (WhitePiece(tmp_image))))
			{
				tmp_image = Table_obj[r-2][c-2]->GetImage();
				if(tmp_image == &b_space_bmp)
				{
					Move_c[Num_moves] = c-2;
					Move_r[Num_moves] = r-2;
					Was_jump[Num_moves] = 1;
					Jump_c[Num_moves] = c-1;
					Jump_r[Num_moves] = r-1;
					Num_moves++;
					ret_val = 1;
				}
			}
		}
		if((c <= 7) && (r >= 2))
		{
			tmp_image = Table_obj[r-1][c+1]->GetImage();
			if(((!Player) && (BlackPiece(tmp_image))) || ((Player) && (WhitePiece(tmp_image))))
			{
				tmp_image = Table_obj[r-2][c+2]->GetImage();
				if(tmp_image == &b_space_bmp)
				{
					Move_c[Num_moves] = c+2;
					Move_r[Num_moves] = r-2;
					Was_jump[Num_moves] = 1;
					Jump_c[Num_moves] = c+1;
					Jump_r[Num_moves] = r-1;
					Num_moves++;
					ret_val = 1;
				}
			}
		}
	}
	tmp_image = Table_obj[r][c]->GetImage();
	if((Player) || (tmp_image == &w_king_bmp)) 
	{
		if((c <= 7) && (r <= 7))
		{
			tmp_image = Table_obj[r+1][c+1]->GetImage();
			if(((!Player) && (BlackPiece(tmp_image))) || ((Player) && (WhitePiece(tmp_image))))
			{
				tmp_image = Table_obj[r+2][c+2]->GetImage();
				if(tmp_image == &b_space_bmp)
				{
					Move_c[Num_moves] = c+2;
					Move_r[Num_moves] = r+2;
					Was_jump[Num_moves] = 1;
					Jump_c[Num_moves] = c+1;
					Jump_r[Num_moves] = r+1;
					Num_moves++;
					ret_val = 1;
				}
			}
		}
		if((c >= 2) && (r <=7 ))
		{
			tmp_image = Table_obj[r+1][c-1]->GetImage();
			if(((!Player) && (BlackPiece(tmp_image))) || ((Player) && (WhitePiece(tmp_image))))
			{
				tmp_image = Table_obj[r+2][c-2]->GetImage();
				if(tmp_image == &b_space_bmp)
				{
					Move_c[Num_moves] = c-2;
					Move_r[Num_moves] = r+2;
					Was_jump[Num_moves] = 1;
					Jump_c[Num_moves] = c-1;
					Jump_r[Num_moves] = r+1;
					Num_moves++;
					ret_val = 1;
				}
			}
		}
	}
	return ret_val;
	
}

U16 CanMove ( U16 Player, U16 c, U16 r)
{
const IMAGE *tmp_image;
U16 ret_val = 0;
						
	tmp_image = Table_obj[r][c]->GetImage();
	if((!Player) || (tmp_image == &b_king_bmp)) 
	{
		if((c >= 1) && (r >= 1))
		{
			tmp_image = Table_obj[r-1][c-1]->GetImage();
			if(tmp_image == &b_space_bmp)
			{
				Move_c[Num_moves] = c-1;
				Move_r[Num_moves] = r-1;
				Was_jump[Num_moves] = 0;
				Num_moves++;
				ret_val = 1;
			}
		}
		if ((c <= 8) && (r >= 1))
		{
			tmp_image = Table_obj[r-1][c+1]->GetImage();
			if(tmp_image == &b_space_bmp)
			{
				Move_c[Num_moves] = c+1;
				Move_r[Num_moves] = r-1;
				Was_jump[Num_moves] = 0;
				Num_moves++;
				ret_val = 1;
			}
		}
	}
	tmp_image = Table_obj[r][c]->GetImage();
	if((Player) || (tmp_image == &w_king_bmp)) 
	{
		if((c <= 8) && (r <= 8))
		{
			tmp_image = Table_obj[r+1][c+1]->GetImage();
			if(tmp_image == &b_space_bmp)
			{
				Move_c[Num_moves] = c+1;
				Move_r[Num_moves] = r+1;
				Was_jump[Num_moves] = 0;
				Num_moves++;
				ret_val = 1;
			}
		}
		if((c >= 1) && (r <=8 ))
		{
			tmp_image = Table_obj[r+1][c-1]->GetImage();
			if(tmp_image == &b_space_bmp)
			{
				Move_c[Num_moves] = c-1;
				Move_r[Num_moves] = r+1;
				Was_jump[Num_moves] = 0;
				Num_moves++;
				ret_val = 1;
			}
		}
	}
	return ret_val;
	
}

U16 CanPlay ( U16 Player, U16 c, U16 r)
{
	CanJump(Player, c, r);
	CanMove(Player, c, r);
	return Num_moves;
}

U16 CanSelectPiece(U16 Player, U16 c, U16 r)
{
const IMAGE *tmp_image;
	tmp_image = Table_obj[r][c]->GetImage();
	if((((!Player) && (WhitePiece(tmp_image))) || ((Player) && (BlackPiece(tmp_image)))) &&
	   (CanPlay(Player, c, r)))
		return 1;
	return 0;
}

void SelectPiece(U16 Player, U16 c, U16 r)
{
const IMAGE *tmp_image;
	if(Sound)
	{
		piezo_play_jingle(6, PIEZO_FLAG_NOQUIET|PIEZO_FLAG_WAIT);
	}

	tmp_image = Table_obj[r][c]->GetImage();
	
	From_c = c;
	From_r = r;
	if(!Player) /* Select */
		if (tmp_image == &w_king_bmp)
			Table_obj[r][c]->SetImage(&w_king_select_bmp);
		else
			Table_obj[r][c]->SetImage(&w_select_bmp);
	else
		if (tmp_image == &b_king_bmp)
			Table_obj[r][c]->SetImage(&b_king_select_bmp);
		else
			Table_obj[r][c]->SetImage(&b_select_bmp);
	Game_state = GS_MOVE;
	sprintf(Title_bar,"%s - Move%s", Color[Player], Link_str);
	f_MainWindow->SetTitle(Title_bar);
}

void MovePiece(U16 i, U16 c, U16 r)
{
const IMAGE *tmp_image;

	if(Sound)
	{
		piezo_play_jingle(6, PIEZO_FLAG_NOQUIET|PIEZO_FLAG_WAIT);
	}

	Mult_jump = 0;
	tmp_image = Table_obj[From_r][From_c]->GetImage();
	if(!Player)	/* Un-Select */
		if (tmp_image == &w_king_select_bmp)
			Table_obj[r][c]->SetImage(&w_king_bmp);
		else
			Table_obj[r][c]->SetImage(&w_man_bmp);
	else
		if (tmp_image == &b_king_select_bmp)
			Table_obj[r][c]->SetImage(&b_king_bmp);
		else
			Table_obj[r][c]->SetImage(&b_man_bmp);
	Table_obj[From_r][From_c]->SetImage(&b_space_bmp);
	if(!Player)	/* King me! */
	{
		if (r == 0)
			Table_obj[r][c]->SetImage(&w_king_bmp);
	}
	else
	{
		if (r == 9)
			Table_obj[r][c]->SetImage(&b_king_bmp);
	}
	if(Was_jump[i]) /* Jump */
	{
		Table_obj[Jump_r[i]][Jump_c[i]]->SetImage(&b_space_bmp);
		if(!Player)
		{
			Num_blacks--;
			if (!Num_blacks)
			{
				Running = 0;
				sprintf(Title_bar,"Game Over, %d moves.%s", Game_score, Link_str);
				f_MainWindow->SetTitle(Title_bar);
				GUI_UpdateNow();
				sprintf(You_won_str, "White won!");
				GUI_Alert(ALERT_INFO, You_won_str);
				f_EnterNameWindow = new EnterNameWindow();
			}
		}
		else
		{
			Num_whites--;
			if (!Num_whites)
			{
				Running = 0;
				sprintf(Title_bar,"Game Over, %d moves.%s", Game_score, Link_str);
				f_MainWindow->SetTitle(Title_bar);
				GUI_UpdateNow();
				sprintf(You_won_str, "Black won!");
				GUI_Alert(ALERT_INFO, You_won_str);
				if(Black_player == EBM)
					f_MainWindow->HighScore(Game_score, "EBM");
				else
					f_EnterNameWindow = new EnterNameWindow();
			}
		}

		sprintf(Black_label_str,"Black pieces - %d", Num_blacks);
		Black_label->SetLabel(Black_label_str);
		sprintf(White_label_str,"White pieces - %d", Num_whites);
		White_label->SetLabel(White_label_str);
	}
	if(Running)
	{
		if((Was_jump[i]) && (CanJump(Player, c, r)))
		{
			Mult_jump = 1;
			From_c = c;
			From_r = r;
			tmp_image = Table_obj[r][c]->GetImage();
			if(!Player)
				if (tmp_image == &w_king_bmp)
					Table_obj[r][c]->SetImage(&w_king_select_bmp);
				else
					Table_obj[r][c]->SetImage(&w_select_bmp);
			else
				if (tmp_image == &b_king_bmp)
					Table_obj[r][c]->SetImage(&b_king_select_bmp);
				else
					Table_obj[r][c]->SetImage(&b_select_bmp);
		}
		else
		{
			Game_score++;
			Player ^=1;
			Game_state = GS_SELECT;
			sprintf(Title_bar,"%s - Select%s", Color[Player], Link_str);
			f_MainWindow->SetTitle(Title_bar);
		}
	}
}


U16 Lookahead( U16 start_move, U16 from_c, U16 from_r , U16 any)
{
U16 r, c;
U16 abort, move, i;
const IMAGE *tmp_image;
U16 to_c, to_r;

	abort = 0;
	From_c = from_c;
	From_r = from_r;
	tmp_image = Table_obj[from_r][from_c]->GetImage();
	for(move = start_move; ((move < Num_moves) && !abort); move++)
	{
		to_c = Move_c[move];
		to_r = Move_r[move];
		Table_obj[to_r][to_c]->SetImage(tmp_image);
		Table_obj[From_r][From_c]->SetImage(&b_space_bmp);
		for( r = 0; ((r < 10) && !abort); r++) 
			for( c = 0; ((c < 10) && !abort); c++)
				if (CanSelectPiece(0, c, r))
					if (any)
					{
				    		if (Was_jump[0])
							abort = 1;
					}
					else
					{
						for (i = 0; i < Num_moves; i++)
							if(Was_jump[i] && (Jump_r[i] == to_r) && (Jump_c[i] == to_c))
								abort = 1;
					}
		tmp_image = Table_obj[to_r][to_c]->GetImage();
		Table_obj[from_r][from_c]->SetImage(tmp_image);
		Table_obj[to_r][to_c]->SetImage(&b_space_bmp);
		if(abort)
			abort = 0;
		else
		{
			abort = 1;
			Do_move = move;
		}
		CanSelectPiece(Player, from_c, from_r);
	}
	return abort;
 }

U16 EbmSelectRange(U16 min_r, U16 max_r, U16 restrict, U16 any)
{
U16 start_c, start_r;
U16 r, c;
U16 abort;
U16 dif, n_dif, dif_r, dif_c;
U16 best_move;
const IMAGE *tmp_image;
	
	start_c = (rand()%1000)/100;
	start_r = ((rand()%((max_r - min_r)*100))/100) + min_r;
	abort = 0;
	r = c = 0;
	r = start_r;
	do
	{
		c = start_c;
		do
            	{
            		if (CanSelectPiece(Player, c, r))
            		{
            			abort = Lookahead(0, c, r, any);
            			tmp_image = Table_obj[From_r][From_c]->GetImage();
            			if(abort && (tmp_image == &b_king_bmp))
				{
					dif = 100;
					best_move = Do_move;
					do
					{
		            			dif_c = abs((int)(Move_c[Do_move]) - (int)(First_c));
        		    			dif_r = abs((int)(Move_r[Do_move]) - (int)(First_r));
            					n_dif = dif_c + dif_r;
            					if(n_dif <= dif)
            					{
            						dif = n_dif;
            						best_move = Do_move;
            					}
		            			abort = Lookahead(++Do_move, c, r, any);
            				}while (abort);
            				Do_move = best_move;
	            			dif_c = abs((int)(From_c) - (int)(First_c));
       		    			dif_r = abs((int)(From_r) - (int)(First_r));
           				n_dif = dif_c + dif_r;
             				if((((First_r > From_r) && (Move_r[best_move] < From_r)) || (n_dif < dif)) &&
             				   (restrict))
            					abort = 0;
					else
	            				abort = 1;
            			}
			}
      			if (++c >= 10)
				c = 0;
		} while ((c != start_c) && !abort);
		if (++r >= max_r)
			r = min_r;
	} while((r != start_r) && !abort);
	return abort;
}

U16 EbmSelect( void )
{
U16 abort, r, c, first_white;
U16 rr, cc;
const IMAGE *tmp_image;

	Do_move = 0;
	abort = 0;
	first_white = 0;
	for( r = 0; ((r < 10) && !abort); r++) /* Can jump? Jump.*/
		for( c = 0; ((c < 10) && !abort); c++)
		{
       			tmp_image = Table_obj[r][c]->GetImage();
			if((!first_white) && (WhitePiece(tmp_image)))
			{
				++first_white;
				First_r = r;
				First_c = c;
			}
			if (CanSelectPiece(Player, c, r) && Was_jump[0])
			{
				From_c = c;
				From_r = r;
				Do_move = 0;
				abort = 1;
			}
		}
	if (!abort)
		for( r = 0; ((r < 10) && !abort); r++) /* Can be jumped?, Move. */
			for( c = 0; ((c < 10) && !abort); c++)
				if (CanSelectPiece(0, c, r) && Was_jump[0])
					for( rr = 0; ((rr < 10) && !abort); rr++) 
						for( cc = 0; ((cc < 10) && !abort); cc++)
							if (CanSelectPiece(Player, cc, rr))
				            			abort = Lookahead(0, cc, rr,1);
	if (!abort)
		for( r = 0; ((r < 10) && !abort); r++) /* Can be jumped?, Move. */
			for( c = 0; ((c < 10) && !abort); c++)
				if (CanSelectPiece(0, c, r) && Was_jump[0])
					if (CanSelectPiece(Player, Jump_c[0], Jump_r[0]))
		            			abort = Lookahead(0, Jump_c[0], Jump_r[0], 0);
	if (!abort)
		for( r = 0; ((r < 10) && !abort); r++) /* Can be jumped?, Move. */
			for( c = 0; ((c < 10) && !abort); c++)
				if (CanSelectPiece(0, c, r) && Was_jump[0])
					if (CanSelectPiece(Player, Jump_c[0], Jump_r[0]))
					{
						From_c = Jump_c[0];
						From_r = Jump_r[0];
						Do_move = 0;
						abort = 1;
					}

	if (!abort)	/* Can be Jumed after move.  Don't move. */
		abort = EbmSelectRange(1, 10, 1, 1);
	if (!abort)	/* Can be Jumed after move.  Don't move. */
		abort = EbmSelectRange(1, 10, 0, 1);
	if (!abort)	/* Can be Jumed after move.  Don't move. */
		abort = EbmSelectRange(0, 1, 0, 1);
	if (!abort)	/* Can be Jumed after move.  Don't move. */
		abort = EbmSelectRange(1, 10, 0, 0);
	if (!abort)	/* Can be Jumed after move.  Don't move. */
		abort = EbmSelectRange(0, 1, 0, 0);
	if (!abort)
		for( r = 0; ((r < 10) && !abort); r++) /* move any */
			for( c = 0; ((c < 10) && !abort); c++)
				if (CanSelectPiece(Player, c, r))
				{
					abort = 1;
					From_c = c;
					From_r = r;
					Do_move = 0;
				}
	return abort;
}


//------------------------------------------------------------------------------
S32 MainWindow::MsgHandler( MSG_TYPE type, CViewable *from, S32 data )
{
U16 x, y;
U16 r, c;
U16 abort, i;
char data_str[100];

 // message handler

	switch ( type )
	{
		case MSG_KEY:
			switch ( data )
			{
				case K_MENU: // process request to show menu
					m_menu1->Show();
					return 1;

				case K_JOG_UP:
				case K_JOG_ENTER:
				case K_JOG_DOWN:
					Sound ^= 1;
					return 1;

				default:
					return 0;
			}
			
		case MSG_LINK_PEN_DOWN:
		case MSG_PEN_DOWN:
            		if ((Running==1) && ((!Player) || (Black_player == HUMAN))) 
            		{
				y = Board->FindRowFromY(data & 0xffff);
				x = Board->FindColFromX((data >> 16) & 0xffff);
				if ((x != 65535) && (y != 65535))
				{
					r = y;
					c = x;
					if(Game_state == GS_SELECT)
					{
						if(CanSelectPiece(Player, c, r))
							SelectPiece(Player, c, r);
					}
					else
					{
						i = 0;
						while ((i < Num_moves) && ((Move_c[i] != c) || (Move_r[i] != r)))
							i++;
						if(i < Num_moves)
						{
							MovePiece(i, c, r);
				            		GUI_EventMessage( (MSG_TYPE)(MSG_GAME_LOOP), from, 0 );
						}
						else if(!Mult_jump)
						{
							const IMAGE *tmp_image;
						
							tmp_image = Table_obj[From_r][From_c]->GetImage();
							if(!Player)	/* Un-Select */
								if (tmp_image == &w_king_select_bmp)
									Table_obj[From_r][From_c]->SetImage(&w_king_bmp);
								else
									Table_obj[From_r][From_c]->SetImage(&w_man_bmp);
							else
								if (tmp_image == &b_king_select_bmp)
									Table_obj[From_r][From_c]->SetImage(&b_king_bmp);
								else
									Table_obj[From_r][From_c]->SetImage(&b_man_bmp);
													
							Game_state = GS_SELECT;
							sprintf(Title_bar,"%s - Select%s", Color[Player], Link_str);
							f_MainWindow->SetTitle(Title_bar);
							if(CanSelectPiece(Player, c, r))
								SelectPiece(Player, c, r);
						}
					}
					GUI_NeedUpdate();
				}
            			if((Link_str) && (type == MSG_PEN_DOWN))
            			{
            				GUI_UpdateNow();
	            			sprintf(data_str, "%ld", data);
        	    			do_write(data_str);
        	    		}
			}
			return 1;
 			
            	case MSG_TIMER:
            		GUI_EventMessage( (MSG_TYPE)(MSG_GAME_LOOP), from, 0 );
            		if(Link_str[0])
	            		do_read();
            		return 1;
            		
            	case MSG_GAME_LOOP:
            		if ((Running==1) && (Player) && (Black_player == EBM)) 
            		{
            			if(Game_state == GS_SELECT)
            			{
            				abort = EbmSelect();
					if(abort)
					{
						SelectPiece(Player, From_c, From_r);
					}
					else
					{
						Running = 0;
						sprintf(Title_bar,"Stalemate%s", Link_str);
						f_MainWindow->SetTitle(Title_bar);
					}
            			}
            			else
            			{
            				MovePiece(Do_move, Move_c[Do_move], Move_r[Do_move]);
            			}
				GUI_NeedUpdate();
            		}
            		for( i = 0; i > 25; i++)
				rand();
			Board->SetTimer(TIMER_SPEED, FALSE, 0);
 			return 1;

		case MSG_BUTTON_REPEAT:
		case MSG_BUTTON_SELECT:
			if (Running)
			{
				switch( from->GetID() )
				{
					default:
						return 0;
				}
  				return (1);
			}
  			return (1);

		case MSG_MENU_SELECT: // handle menu selection
		{
			switch ((U16)data)
			{
				case IDM_OPTION1: // New Game
					NewGame();
					break;
				case IDM_OPTION2: // High scores
					HighScore(0, 0);
					break;
				case IDM_OPTION3: // Help
					GUI_TextWindow(1,"Help",Help_text);
					break;
				case IDM_OPTION4: // Handwriting help
					GUI_HandwritingHelp(0);
					break;
				case IDM_OPTION5: // EBM/Human
					if(Black_player == HUMAN)
					{
						Black_player = EBM;
						m_menu1->SetRow( 4,IDM_OPTION5,"Black to human", 'p');
					}
					else
					{
						Black_player = HUMAN;
						m_menu1->SetRow( 4,IDM_OPTION5,"Black to EBM", 'p');
					}
					break;
				case IDM_OPTION6: // Link
					do_open();
					if(Link_str)
					{
						Black_player = HUMAN;
						m_menu1->SetRow( 4,IDM_OPTION5,"Black to EBM", 'p');
					}
					NewGame();
					break;
				case IDM_OPTION7: // Exit
					GUI_Exit();
					break;

			}
			return 1;
		}

		default:
			return 0;
	}
				
  return CWindow::MsgHandler(type, from, data);
}

//------------------------------------------------------------------------------
void MainWindow::Initialize(void)
{
	U16 c, r;
	U16 Column_size;

// set up menu #1 - with shortcuts
	m_menu1=new CMenu(ID_MENU1,0,32767,0,this);
	m_menu1->SetNumRows(8);
	int row=-1;
	m_menu1->SetRow( ++row,IDM_OPTION1,"New Game",'n' );
	m_menu1->SetRow( ++row,IDM_OPTION2,"High Scores",'s' );
	m_menu1->SetRow( ++row,IDM_OPTION3,"Help",'h' );
	m_menu1->SetRow( ++row,IDM_OPTION4,"Handwriting Help",'w' );
	Black_player = EBM;
	m_menu1->SetRow( ++row,IDM_OPTION5,"Black to human", 'p');
	m_menu1->SetRow( ++row,IDM_OPTION6,"Link", 'l');
	m_menu1->SetSeparatorRow(++row);
	m_menu1->SetRow( ++row,IDM_OPTION7,"Exit",'e' );
	
// do additional setup functions

	Num_rows = 10;
	Num_columns = 10;
	Num_players = 2;
	Mult_jump = 0;

	Board=new CTable(MainWindow_Board,193,183, 7);
	AddChild(Board,1,20);

	Column_size = 196/(Num_columns);
	for (c = 0; c < (Num_columns-1); c++)
		Board->AddColumn(Column_size);
	Board->AddColumn(0);
	Board->SetNumRows(Num_rows);
	
	for (r = 0; r < Num_rows; r++)
	{
		for (c = 0; c < Num_columns; c++)
		{
			Table_obj[r][c]=new CIcon(MainWindow_Board+(r*c)+c,&w_space_bmp);
			Board->SetCell( Table_obj[r][c], r, c );
		}
	}

	Black_label=new CLabel(MainWindow_Black_label,"Black pieces",0);
	AddChild(Black_label,40,5);
	White_label=new CLabel(MainWindow_White_label,"White pieces",0);
	AddChild(White_label,40,207);

	EnableOSNotify();
	rand();
	
	NewGame();
	Board->SetTimer(TIMER_SPEED, FALSE, 0);
	
	Sound = 1;

}


//------------------------------------------------------------------------------
MainWindow::~MainWindow(void)
{
	//delete m_menu1; // cleanup
}

//------------------------------------------------------------------------------
S32 GUI_main( MSG_TYPE type, CViewable *object, S32 data )
{
	switch (type)
	{
	case MSG_APP_START:
		if (OS_is_present && hostIO_am_I_the_current_task())
			HOSTIO_INLINE_BREAKPOINT();
		f_MainWindow = new MainWindow();
		f_MainWindow->Initialize(); // do additional setup here
		return 1;
	case MSG_APP_STOP: // prog exit & cleanup
		//delete f_MainWindow;
		do_close();
		return 1;
	case MSG_OS_NOTIFY:
		switch ((U16) data)
		{
			case NTFY_POWER_ON:
				return 0;
			case NTFY_FOREGROUND:
				if(Running == 2)
					Running = 1;
				return 1;
			case NTFY_BACKGROUND:
				if(Running)
					Running = 2;
				return 1;
			default:
				return 0;
		}
	default:
		break;
	}
	return 0;
};
//------------------------------------------------------------------------------


